@@ -1,40 +1,21 @@
-/***************************************************************************
- *   Copyright 2001-2009 Anne-Marie Mahfouf <annma@kde.org>                *
- *   Copyright 2014 Rahul Chowdhury <rahul.chowdhury@kdemail.net>          *
- *                                                                         *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
-
+// SPDX-FileCopyrightText: 2001-2009 Anne-Marie Mahfouf <annma@kde.org>
+// SPDX-FileCopyCopyright: 2014 Rahul Chowdhury <rahul.chowdhury@kdemail.net>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "khangman.h"
 
 #include <QApplication>
 #include <QStandardPaths>
-#include <QQuickWidget>
 #include <QQmlContext>
 #include <QQmlEngine>
+#include <QShortcut>
+#include <QDesktopServices>
 
-#include <KLocalizedContext>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KRandom>
 #include <knewstuff_version.h>
-#include <KNSWidgets/Dialog>
-#include <KHelpMenu>
+#include <KAuthorized>
 
 #include <KEduVocDocument>
 #include <keduvocexpression.h>
@@ -42,39 +23,40 @@
 
 #include "langutils.h"
 #include "prefs.h"
-#include <QShortcut>
+
+namespace
+{
+/**
+ * Strip the accents off given string
+ * @params original string to strip accents off of
+ * @returns string without accents
+ */
+QString stripAccents(const QString &original)
+{
+    QString noAccents;
+    QString decomposed = original.normalized(QString::NormalizationForm_D);
+    for (int i = 0; i < decomposed.length(); ++i) {
+        if ( decomposed[i].category() != QChar::Mark_NonSpacing ) {
+            noAccents.append(decomposed[i]);
+        }
+    }
+    return noAccents;
+}
+
+}
 
 KHangMan::KHangMan()
-        : QMainWindow(),
+        : QObject (),
           m_currentCategory(0),
           m_currentLanguage(0),
           m_winCount(0),
           m_lossCount(0),
           m_randomInt(0),
           m_scoreMultiplyingFactor(1),
-          m_netScore(0),
-          m_helpMenu(new KHelpMenu(nullptr))
+          m_netScore(0)
 {
     setObjectName(QStringLiteral("KHangMan"));
 
-    m_view = new QQuickWidget(this);
-    m_view->rootContext()->setContextProperty(QStringLiteral("khangman"), this);
-
-    // prepare i18n
-    auto context = new KLocalizedContext(this);
-    m_view->engine()->rootContext()->setContextObject(context);
-
-    KConfigGroup windowConfig = config(QStringLiteral("Window"));
-    if (windowConfig.hasKey("geometry")) {
-        setGeometry(windowConfig.readEntry<QRect>("geometry", QRect()));
-        setWindowState(Qt::WindowState(windowConfig.readEntry("windowState").toInt()));
-    }
-
-    setMinimumSize(800, 600);
-
-    m_view->setResizeMode(QQuickWidget::SizeRootObjectToView);
-
-    setCentralWidget(m_view);
     scanLanguages();
 
     //find the themes
@@ -82,31 +64,21 @@ KHangMan::KHangMan()
     
     QShortcut *quitShortcut = new QShortcut(QKeySequence::Quit, this);
     connect(quitShortcut, &QShortcut::activated, qApp, &QCoreApplication::quit);
+
+    loadLevels();
+    // kvtml files have been found
+
+    if (m_themeFactory.getQty() == 0) { // themes not present
+        Q_EMIT errorOccured(i18n("No theme files found."));
+        exit(EXIT_FAILURE);
+    }
 }
 
-KHangMan::~KHangMan()
+void KHangMan::showHandbook() const
 {
-    KConfigGroup windowConfig = config(QStringLiteral("Window"));
-    windowConfig.writeEntry("geometry", geometry());
-    windowConfig.writeEntry("windowState", int(windowState()));
-
-    delete m_view;
-    m_view = nullptr;
-}
-
-void KHangMan::showAboutKHangMan()
-{
-    m_helpMenu->aboutApplication();
-}
-
-void KHangMan::showAboutKDE()
-{
-    m_helpMenu->aboutKDE();
-}
-
-void KHangMan::showHandbook()
-{
-    m_helpMenu->appHelpActivated();
+    if (KAuthorized::authorizeAction(QStringLiteral("help_contents"))) {
+        QDesktopServices::openUrl(QUrl(QStringLiteral("help:/")));
+    }
 }
 
 KConfigGroup KHangMan::config(const QString &group)
@@ -118,9 +90,13 @@ KConfigGroup KHangMan::config(const QString &group)
 //                               Slots
 void KHangMan::setCurrentCategory(int index)
 {
-    QMap<QString, QString>::const_iterator currentLevel = m_titleLevels.constBegin() + index;
+    if (m_titleLevels.keys().count() <= index) {
+        return;
+    }
+    const auto key = m_titleLevels.keys()[index];
+
     Prefs::setCurrentLevel(index);
-    Prefs::setLevelFile(currentLevel.value());
+    Prefs::setLevelFile(m_titleLevels[key]);
     Prefs::self()->save();
     m_currentCategory = index;
     Q_EMIT currentCategoryChanged();
@@ -152,11 +128,10 @@ void KHangMan::readFile()
     myFile.setFileName(Prefs::levelFile());
 
     if (!myFile.exists()) {
-        QString  mString = i18n("File $KDEDIR/share/apps/kvtml/%1/%2 not found.\n"
+        Q_EMIT errorOccured(i18n("File $KDEDIR/share/apps/kvtml/%1/%2 not found.\n"
             "Please check your installation.",
             Prefs::selectedLanguage(),
-            Prefs::levelFile());
-        KMessageBox::error (this, mString, i18n("Error"));
+            Prefs::levelFile()));
     }
 
     // Detects if file is a kvtml file so that it's a hint enable file
@@ -167,6 +142,7 @@ void KHangMan::nextWord()
 {
     // If there are no words, there's nothing we can do, trying will crash
     if (m_randomList.isEmpty()) {
+        Q_EMIT errorOccured(i18n("No word list loaded"));
         return;
     }
 
@@ -221,18 +197,6 @@ void KHangMan::slotSetWordsSequence()
     KRandom::shuffle(m_randomList);
 }
 
-QString KHangMan::stripAccents(const QString & original)
-{
-    QString noAccents;
-    QString decomposed = original.normalized(QString::NormalizationForm_D);
-    for (int i = 0; i < decomposed.length(); ++i) {
-        if ( decomposed[i].category() != QChar::Mark_NonSpacing ) {
-            noAccents.append(decomposed[i]);
-        }
-    }
-    return noAccents;
-}
-
 void KHangMan::replaceLetters(const QString& charString)
 {
     QChar ch = charString.at(0);
@@ -266,7 +230,7 @@ void KHangMan::calculateNetScore()
     Q_EMIT netScoreChanged();
 }
 
-bool KHangMan::containsChar(const QString &sChar)
+bool KHangMan::containsChar(const QString &sChar) const
 {
     return m_originalWord.contains(sChar) || stripAccents(m_originalWord).contains(sChar);
 }
@@ -282,7 +246,7 @@ void KHangMan::setResolveTime(int resolveTime)
     Q_EMIT resolveTimeChanged();
 }
 
-bool KHangMan::soundEnabled()
+bool KHangMan::soundEnabled() const
 {
     return Prefs::sound();
 }
@@ -293,7 +257,7 @@ void KHangMan::setSoundEnabled(bool sound)
     Q_EMIT soundEnabledChanged();
 }
 
-QStringList KHangMan::languages()
+QStringList KHangMan::languages() const
 {
     return m_languageNames;
 }
@@ -339,32 +303,32 @@ int KHangMan::netScore() const
     return m_netScore;
 }
 
-int KHangMan::currentLanguage()
+int KHangMan::currentLanguage() const
 {
     return m_currentLanguage;
 }
 
-QStringList KHangMan::themes()
+QStringList KHangMan::themes() const
 {
     return m_themeFactory.themeList();
 }
 
-int KHangMan::currentTheme()
+int KHangMan::currentTheme() const
 {
     QStringList themes = m_themeFactory.getNames();
     return themes.indexOf(Prefs::theme());
 }
 
-QString KHangMan::backgroundUrl()
+QUrl KHangMan::backgroundUrl() const
 {
     const KHMTheme *theme = m_themeFactory.getTheme(currentTheme());
     if (theme) {
-        return QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("themes/") + theme->svgFileName());
+        return QUrl::fromLocalFile(QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("themes/") + theme->svgFileName()));
     }
-    return QString();
+    return {};
 }
 
-QColor KHangMan::currentThemeLetterColor()
+QColor KHangMan::currentThemeLetterColor() const
 {
     const KHMTheme *theme = m_themeFactory.getTheme(currentTheme());
     if (theme) {
@@ -375,12 +339,12 @@ QColor KHangMan::currentThemeLetterColor()
     return QColor("white");
 }
 
-QStringList KHangMan::categories()
+QStringList KHangMan::categories() const
 {
     return m_titleLevels.keys();
 }
 
-int KHangMan::currentCategory()
+int KHangMan::currentCategory() const
 {
     return m_currentCategory;
 }
@@ -402,11 +366,6 @@ QString KHangMan::getCurrentHint() const
     return m_hint;
 }
 
-QQmlEngine* KHangMan::getEngine()
-{
-    return m_view->engine();
-}
-
 QStringList KHangMan::alphabet() const
 {
     QStringList letterList;
@@ -457,23 +416,6 @@ void KHangMan::scanLanguages()
     Q_EMIT languagesChanged();
 }
 
-void KHangMan::show()
-{
-    loadLevels();
-    // kvtml files have been found
-
-    if (m_themeFactory.getQty() == 0) { // themes not present
-        KMessageBox::error(this, i18n("No theme files found."), i18n("Error"));
-        exit(EXIT_FAILURE);
-    }
-
-    QMainWindow::show();
-    // add the qml view as the main widget
-    QString location = QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("qml/main.qml"));
-    QUrl url = QUrl::fromLocalFile(location);
-    m_view->setSource(url);
-}
-
 void KHangMan::loadLevels()
 {
     //build the Level combobox menu dynamically depending of the data for each language
@@ -485,7 +427,7 @@ void KHangMan::loadLevels()
         Prefs::self()->save();
         levelFilenames = SharedKvtmlFiles::fileNames(Prefs::selectedLanguage());
         if (levelFilenames.isEmpty()) {
-            KMessageBox::error(this, i18n("No kvtml files found."), i18n("Error"));
+            Q_EMIT errorOccured(i18n("No kvtml files found."));
             exit(EXIT_FAILURE);
         }
     }
@@ -506,23 +448,13 @@ void KHangMan::loadLevels()
     loadLanguageSpecialCharacters();
 }
 
-void KHangMan::slotDownloadNewStuff()
+void KHangMan::slotDownloadNewStuff(KNSCore::Entry *entry)
 {
-    KNSWidgets::Dialog *dialog = new KNSWidgets::Dialog(QStringLiteral("khangman.knsrc"), this);
-    dialog->open();
-
-    connect(dialog, &KNSWidgets::Dialog::finished, this, [this, dialog] {
-        const QList<KNSCore::Entry> entries = dialog->changedEntries();
-        if ( !entries.isEmpty() ){
-            SharedKvtmlFiles::sortDownloadedFiles();
-            //look for languages dirs installed
-            scanLanguages();
-            //refresh Languages menu
-            setCurrentLanguage(m_languages.indexOf(Prefs::selectedLanguage()));
-        }
-
-        dialog->deleteLater();
-    });
+    SharedKvtmlFiles::sortDownloadedFiles();
+    //look for languages dirs installed
+    scanLanguages();
+    //refresh Languages menu
+    setCurrentLanguage(m_languages.indexOf(Prefs::selectedLanguage()));
 }
 
 void KHangMan::loadLanguageSpecialCharacters()
@@ -550,8 +482,6 @@ void KHangMan::loadLanguageSpecialCharacters()
         }
     }
 
-    update();
-
     // We open the file and store info into the stream...
     QFile openFileStream(langFile.fileName());
     openFileStream.open(QIODevice::ReadOnly);
@@ -560,7 +490,7 @@ void KHangMan::loadLanguageSpecialCharacters()
 
     // m_specialCharacters contains all the words from the file
     // FIXME: Better name
-    m_specialCharacters = readFileStr.readAll().split(QLatin1Char('\n'));
+    m_specialCharacters = readFileStr.readAll().split(QLatin1Char('\n'), Qt::SkipEmptyParts);
     openFileStream.close();
 }
 
